using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using RDH.Data.Models;

//:IRegable ,IBaseBLL<Item>,ISnapshotBLL<Item>
namespace RDH.Data.BLL
{
    public partial interface IItemBLL:IRegable ,IBaseBLL<Item>,ISnapshotBLL<Item>
    {
        IEnumerable<Item> ListItems(string filter, int index, int size);
        Item GetByItemId(string itemId);
        Item GetByItemCurrentId(string itemId);
        Item GetIsActiveItem(Item _itemKey);
        int GetCount(string filter);
        IEnumerable<Item> ListItemsWithConfigCount(DispensingDevice device, String filter);
    }
}
